const fetch = require('node-fetch')
const fs = require('./fs-x')
const path = require('path')

module.exports = class Node {
  /**
   * @param {string} url
   * @return {string}
   */
  static conformURL (url) {
    return url[0] === '=' ? `https://raw.githubusercontent.com/github/gitignore/master/${url.substr(1)}.gitignore` : url
  }

  /**
   * @param {string} url
   * @param {string} parent
   * @param {boolean} skipHeader
   * @return {Promise<string>}
   */
  static getContents (url, { parent = '.', skipHeader = false } = {}) {
    if (url.startsWith('fs://')) {
      console.log('Warning: "fs://" is deprecated - Use "file://" instead.')
      url = 'file' + url.substr(2)
    }
    if (url.startsWith('file://')) {
      url = path.join(parent.startsWith('file://') ? parent.substr(7) : parent, url.substr(7))
      return fs.readFilePromised(url).then(contents => {
        return skipHeader === true ? contents : `## getignore: ${url} ##\n${contents}`
      }).catch(_ => {
        console.log(`Error:   ${url}`)
        return `## getignore: error: ${url} ##\n`
      })
    } else if (url.startsWith('http://') || url.startsWith('https://')) {
      return fetch(url).then(res => {
        if (res.ok) {
          console.log(`Success: ${url}`)
          return res.text().then(contents => skipHeader === true ? contents : `## getignore: ${url} ##\n${contents}`)
        } else {
          console.log(`Error:   ${url}`)
          return `## getignore: error: ${url} ##\n`
        }
      })
    } else {
      console.log(`Error:   ${url}`)
      return Promise.resolve(`## getignore: error: ${url} ##\n`)
    }
  }

  /**
   * @param {string} url
   * @param {string} parent
   * @param {boolean} skipHeader
   * @return {Promise<string>}
   */
  static evaluate (url, { parent = '.', skipHeader = false } = {}) {
    return Node.getContents(Node.conformURL(url), { parent: parent, skipHeader: skipHeader })
      .then(_ => _.split('\n'))
      .then(_ => Promise.all(_.map(line => line.startsWith('#=') ? Node.evaluate(line.substr(2), { parent: Node.getParent(url) }) : line)))
      .then(_ => _.join('\n'))
      .then(Node.ensureSingleNewLineAtEnd)
  }

  /**
   * @param {string} contents
   * @return {string}
   */
  static ensureSingleNewLineAtEnd (contents) {
    if (contents.endsWith('\n\n')) {
      return Node.ensureSingleNewLineAtEnd(contents.slice(0, -1))
    } else if (contents.endsWith('\n')) {
      return contents
    } else {
      return contents + '\n'
    }
  }

  /**
   * @param {string} url
   * @return {string}
   */
  static getParent (url) {
    return url.startsWith('file://') ? 'file://' + path.dirname(url.substr(7)) : path.dirname(url)
  }
}
