/* eslint-env mocha */
/* eslint-disable no-unused-expressions */
const chai = require('chai')
var chaiAsPromised = require('chai-as-promised')
chai.use(chaiAsPromised)
const expect = chai.expect
const fs = require('fs')
const mock = require('mock-fs')
const stdMocksWrapper = require('../support/std-mocks-wrapper')

const cli = require('../../lib/cli')

describe('cli', function () {
  it('should fail when .getignore is missing', function () {
    mock()

    return stdMocksWrapper(() => cli()).then(_ => {
      const exists = fs.existsSync('.gitignore')
      mock.restore()

      expect(_.stdout).to.eql(['Error: File .getignore not found!\n'])
      expect(_.stderr).to.be.empty
      expect(exists).to.be.false
    })
  })

  it('should evaluate existing .getignore, skip first header and end with a new line', function () {
    mock({
      '.getignore': 'x\n#=file://test.gitignore',
      'test.gitignore': 'y'
    })

    return cli().then(_ => {
      const exists = fs.existsSync('.gitignore')
      const contents = fs.readFileSync('.gitignore').toString()
      mock.restore()

      expect(exists).to.be.true
      expect(contents).to.equal('x\n## getignore: test.gitignore ##\ny\n')
    })
  })
})
