/* eslint-env mocha */
const chai = require('chai')
var chaiAsPromised = require('chai-as-promised')
chai.use(chaiAsPromised)
const expect = chai.expect
const mock = require('mock-fs')
const fs = require('fs')

const writeFilePromised = require('../../lib/write-file-promised')

describe('writeFilePromised', function () {
  afterEach(function () {
    mock.restore()
  })

  it('should write and resolve when done', function (done) {
    mock()
    writeFilePromised('test.txt', 'some file contents').then(_ => {
      fs.readFile('test.txt', (err, data) => {
        if (err) {
          done(err)
        } else {
          expect(data.toString()).to.equal('some file contents')
          done()
        }
      })
    })
  })

  it('should reject with error', function (done) {
    fs.writeFile('', 'some file contents', err => {
      expect(writeFilePromised('', 'some file contents').catch(err => err.message)).to.become(err.message)
      done()
    })
  })
})
